package com.reactlibrary;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

public class AlbumsDialogFragment extends androidx.fragment.app.DialogFragment {

    private static final String BUCKET_NAMES_ARG = "BUCKET_NAME_ARGS";
    private static final String BUCKET_THUMBNAIL_ARGS = "BUCKET_THUMBNAIL_ARGS";
    private static final String LANGUAGE_CODE = "LANGUAGE_CODE";

    private RecyclerView mAlbumsRecyclerView;
    private List<Bucket> buckets;
    private String thumbnailPath;
    private String languageCode;

    private AlbumChosen caller;
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        buckets = (ArrayList<Bucket>) getArguments().getSerializable(BUCKET_NAMES_ARG);
        thumbnailPath = getArguments().getString(BUCKET_THUMBNAIL_ARGS);
        caller = (PickerActivity) getActivity();
        languageCode = getArguments().getString(LANGUAGE_CODE);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        LayoutInflater inflater = LayoutInflater.from(getActivity());

        View v = inflater.inflate(R.layout.dialog_albums, null);

        mAlbumsRecyclerView = v.findViewById(R.id.albums_recycler_view);
        mAlbumsRecyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        mAlbumsRecyclerView.setAdapter(new AlbumsAdapter());

        final Dialog dialog = new AlertDialog.Builder(getActivity())
                .setView(v)
                .setNegativeButton(languageCode.equals("ar") ? "إلغاء" : "Cancel", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dismiss();
                    }
                })
                .create();

        return dialog;
    }

    public class AlbumsAdapter extends RecyclerView.Adapter<AlbumsAdapter.AlbumsViewHolder> {
        @NonNull
        @Override
        public AlbumsViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            LayoutInflater layoutInflater = LayoutInflater.from(getActivity());
            View v = layoutInflater.inflate(R.layout.albums_view_holder, parent, false);
            AlbumsViewHolder albumsViewHolder = new AlbumsViewHolder(v);

            return albumsViewHolder;
        }

        @Override
        public void onBindViewHolder(@NonNull AlbumsViewHolder holder, int position) {
            if(position != 0){
                holder.setBucket(buckets.get(position-1));
            } else {
                holder.setBucket(new Bucket(null, thumbnailPath));
            }
            holder.updateItem();
        }

        @Override
        public int getItemCount() {
            return buckets.size() + 1;
        }

        public class AlbumsViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
            private Bucket bucket;
            private ImageView albumsImageView;
            private TextView albumsTextView;

            public AlbumsViewHolder(View itemView) {
                super(itemView);
                itemView.setOnClickListener(this);
                albumsImageView = itemView.findViewById(R.id.album_image_view);
                albumsTextView = itemView.findViewById(R.id.album_text_view);
            }
            public void updateItem(){
                Picasso.get().load("file://"+bucket.getThumbnailPath()).fit().into(albumsImageView);
                if(bucket.getBucketName() != null){
                    albumsTextView.setText(bucket.getBucketName());
                } else {
                    albumsTextView.setText(languageCode.equals("ar") ? "كل الصور" : "All Pictures");
                }
            }
            public void setBucket(Bucket bucket){
                this.bucket = bucket;
            }

            @Override
            public void onClick(View view) {
                caller.chosenAlbum(bucket.getBucketName());
            }
        }


    }

    public static AlbumsDialogFragment getInstance(ArrayList<Bucket> bucketNames, String thumbnailPath, String languageCode){
        AlbumsDialogFragment albumsDialogFragment = new AlbumsDialogFragment();

        Bundle args = new Bundle();
        args.putSerializable(BUCKET_NAMES_ARG, bucketNames);
        args.putString(BUCKET_THUMBNAIL_ARGS, thumbnailPath);
        args.putString(LANGUAGE_CODE, languageCode);
        albumsDialogFragment.setArguments(args);

        return albumsDialogFragment;
    }

    public interface AlbumChosen{
        public void chosenAlbum(String bucketName);
    }
}
