package com.reactlibrary;

import android.content.Context;
import android.database.Cursor;
import android.provider.MediaStore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ImageManager {
    private static ImageManager instance;
    private Set<Image> selectedImages;

    private Map<String, List<String>> imagePathsMap; // key = Bucket name, value = list of paths of all images that belong to that bucket;

    private Context mImageManagerContext;

    private ImageManager(Context context){
        this.mImageManagerContext = context;
    }

    public static synchronized ImageManager getInstance(Context context){
        if(instance != null){
            return instance;
        } else {
            return new ImageManager(context);
        }
    }

    public List<String> getImagePathsFromBucket(Context context, String bucketName){
        List<String> bucketImagePaths = new ArrayList<>();
        String[] queryColumns = {MediaStore.Images.Media.DATA, MediaStore.Images.Media.DATE_ADDED};
        String[] queryArgs = new String[] {bucketName};
        final Cursor cursor = context.getContentResolver().
                query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,queryColumns
                        , MediaStore.Images.Media.BUCKET_DISPLAY_NAME + " like ?"
                        ,queryArgs
                        , MediaStore.Images.Media.DATE_ADDED + " DESC");
        int imagePathColumnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);

        if(cursor.moveToFirst()){
            do{
                String imagePath = cursor.getString(imagePathColumnIndex);
                bucketImagePaths.add(imagePath);
            }while(cursor.moveToNext());
        }

        return bucketImagePaths;
    }

    public List<Bucket> getAllBuckets(Context context){
        List<Bucket> buckets = new ArrayList<>();
        String[] queryColumns = {MediaStore.Images.Media.BUCKET_DISPLAY_NAME};
        final Cursor cursor = context.getContentResolver().
                query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,queryColumns
                        ,null
                        ,null
                        , null);
        int bucketIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.BUCKET_DISPLAY_NAME);

        if(cursor.moveToFirst()){
            do{
                String bucketName = cursor.getString(bucketIndex);
                Bucket bucket = new Bucket(bucketName, null);
                boolean isInList = false;
                for(Bucket b: buckets){
                    if(b.getBucketName().equals(bucket.getBucketName())){
                        isInList = true;
                        break;
                    }
                }

                if(!isInList){
                    buckets.add(bucket);
                }
            }while(cursor.moveToNext());
        }

        return buckets;
    }

    public String getThumbnailPathOfAllImagesAlbum(Context context){
        List<String> imagePaths = getAllImagePaths(context);

        return imagePaths.get(0);
    }

    public Map<String, List<String>> getAllImagePathsRelativeToBucket(Context context){
        Map<String, List<String>> imagePathsMap = new HashMap<>();

        String[] queryColumns = {MediaStore.Images.Media.DATA, MediaStore.Images.Media.BUCKET_DISPLAY_NAME, MediaStore.Images.Media.DATE_ADDED};

        final Cursor cursor = context.getContentResolver().
                query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,queryColumns
                        ,null
                        ,null
                        , MediaStore.Images.Media.DATE_ADDED + " DESC");

        int[] columnsIndecies = new int[] {cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA),  cursor.getColumnIndexOrThrow(MediaStore.Images.Media.BUCKET_DISPLAY_NAME)};
        if(cursor.moveToFirst()){
            do{
                String imagePath = cursor.getString(columnsIndecies[0]);
                String imageBucketName = cursor.getString(columnsIndecies[1]);

                if(imagePathsMap.containsKey(imageBucketName)){
                    imagePathsMap.get(imageBucketName).add(imagePath);
                } else {
                    imagePathsMap.put(imageBucketName, new ArrayList<String>());
                    imagePathsMap.get(imageBucketName).add(imagePath);
                }
            }while(cursor.moveToNext());
        }

        return imagePathsMap;
    }

    public Map<String, List<Image>> getImageObjectsMap(Context context){
        Map<String, List<Image>> imageObjectsMap = new HashMap<>();
        Map<String, List<String>> imagePathsMap = getAllImagePathsRelativeToBucket(context);

        for(String bucketName: imagePathsMap.keySet()){
            List<String> imagePaths = imagePathsMap.get(bucketName);
            for (String imagePath: imagePaths){
                Image image = new Image(imagePath, false);

                if(imageObjectsMap.containsKey(bucketName)){
                    imageObjectsMap.get(bucketName).add(image);
                } else {
                    imageObjectsMap.put(bucketName, new ArrayList<Image>());
                    imageObjectsMap.get(bucketName).add(image);
                }
            }
        }

        return imageObjectsMap;
    }

    public List<Image> getImageObjectsFromPaths(Context context, List<String> paths){
        List<String> imagePaths = paths;
        List<Image> images = new LinkedList<>();

        for(String path: imagePaths){
            Image image = new Image(path, false);
            images.add(image);
        }

        return images;
    }

    /*TODO: Check the possibility of removing this*/
    public List<String> getAllImagePaths(Context context){
        String[] queryColumns = {MediaStore.Images.Media.DATA, MediaStore.Images.Media.DATE_ADDED};

        final Cursor cursor = context.getContentResolver().
                query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,queryColumns
                        ,null
                        ,null
                        , MediaStore.Images.Media.DATE_ADDED + " DESC");

        List<String> imagePaths = new ArrayList<>(cursor.getCount());
        int columnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Thumbnails.DATA);

        if(cursor.moveToFirst()) {
            do {
                imagePaths.add(cursor.getString(columnIndex));
            } while (cursor.moveToNext());
        }

        return imagePaths;
    }

    public List<Bucket> getAllBuckets(Map<String, List<String>> pathMap){
        List<Bucket> buckets = new ArrayList<>();

        for(String bucketName: pathMap.keySet()){
            Bucket bucket = new Bucket(bucketName, pathMap.get(bucketName).get(0));
            buckets.add(bucket);
        }

        return buckets;
    }

    //return the path to the first image in the bucket
    public String getPathOfThumbnailOfBucket(List<String> bucketImagesList){
        return bucketImagesList.get(0);
    }

    public String getFirstImagePathFromBucket(Context context, String bucketName){
        String imagePath = null;

        String[] queryColumns = {MediaStore.Images.Media.DATA, MediaStore.Images.Media.DATE_ADDED};

        final Cursor cursor = context.getContentResolver().
                query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                        ,queryColumns
                        ,MediaStore.Images.Media.BUCKET_DISPLAY_NAME + " like ?"
                        ,new String[]{bucketName}
                        , MediaStore.Images.Media.DATE_ADDED + " DESC" + " LIMIT 1");
        int columnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Thumbnails.DATA);

        if(cursor.moveToFirst()){
            imagePath = cursor.getString(columnIndex);
        }

        return imagePath;
    }

    public List<String> getAllImagePaths(Map<String, List<String>> imagePathsMap) {
        List<String> imagePaths = new ArrayList<>();

        for (String bucketName : imagePathsMap.keySet()) {
            List<String> imagePathRelativeToBucketList = imagePathsMap.get(bucketName);
            imagePaths.addAll(imagePathRelativeToBucketList);
        }

        return imagePaths;
    }

    public Set<Image> getSelectedImages(){
        return selectedImages;
    }

    public void addSelectedImage(Image image){
        selectedImages.add(image);
        System.out.println(getSelectedImageCount());
    }

    public void removeUnselectedImage(Image image){
        selectedImages.remove(image);
    }

    public int getSelectedImageCount(){
        return selectedImages.size();
    }

    public List<Image> getAllImageObjectsFromObjectsMap(Map<String, List<Image>> map){
        List<Image> images = new ArrayList<>();

        for(String bucketName: map.keySet()){
            List<Image> imageList = map.get(bucketName);
            images.addAll(imageList);
        }

        return images;
    }
}
