package com.reactlibrary;
import android.Manifest;
import android.app.Activity;


import android.app.DialogFragment;
import android.app.FragmentManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.os.AsyncTask;
import android.os.Build;
import android.provider.MediaStore;
import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.squareup.picasso.Picasso;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.zip.Inflater;


public class PickerActivity extends AppCompatActivity implements BackgroundProcess, AlbumsDialogFragment.AlbumChosen {

    private ImageManager imageManager;
    private ImageCaptureManager captureManager;

    private Menu imageSelectionMenu;

    private RecyclerView recyclerView;
    private ImagePickerAdapter mImagePickerAdapter;
    private Button mChoseAlbumsButton;

    private AlbumsDialogFragment albumsDialogFragment;

    private ProgressBar progressBar;

    private List<Image> imageObjects;
    private Map<String, List<Image>> imageObjectsMap;
    private List<String> allImagePaths;
    private List<Bucket> buckets;

    private Set<Image> selectedImages;

    private final int WRITE_READ_PERMISSION_REQUEST_CODE = 666;
    private final int CAMERA_PERMISSION_REQUEST_CODE = 667;

    private static final String RESULT = "RESULT";
    private static final String RESULT_GEOLOCATION = "RESULT_GEOLOCATION";
    private static final String WIDTH = "WIDTH";
    private static final String HEIGHT = "HEIGHT";
    private static final String RATIO = "RATIO";
    private static final String MAX_NUMBER_OF_PICTURES = "MAX";
    private static final String LANGUAGE_CODE = "LANGUAGE_CODE";
    private static final String DO_WATERMARK = "DO_WATERMARK";

    private int MAX_NUMBER_OF_SELECTED_IMAGES;

    private int width;
    private int height;
    private int compressionRatio;
    private String languageCode;
    private boolean doWatermark;

    @Override
    public void onBackgroundTaskFinished(List<String> paths) {
        this.sendResultsBack(paths);
    }

    @Override
    public void chosenAlbum(String bucketName) {
        updateUiWithBucket(bucketName);
        albumsDialogFragment.dismiss();
        albumsDialogFragment = null;
    }

    private class ImageProcessingAsyncTask extends AsyncTask<ImageProcessingParams, Void, List<String>> {
        BackgroundProcess caller;

        ImageProcessingAsyncTask(BackgroundProcess caller){
            this.caller = caller;
        }

        @Override
        protected List<String> doInBackground(ImageProcessingParams... imageProcessingParams) {
            ImageProcessingParams params = imageProcessingParams[0];
            List<String> paths = PictureUtils.processImages(PickerActivity.this, params.getWidth(), params.getHeight(), params.getRatio(), params.getPaths(), params.isDoWatermark());

            return paths;
        }

        @Override
        protected void onPostExecute(List<String> paths) {
            super.onPostExecute(paths);
            progressBar.setVisibility(View.GONE);
            caller.onBackgroundTaskFinished(paths);
        }
    }

    private void openCamera(){
        Intent intent = null;

        try {
            intent = captureManager.dispatchTakePictureIntent();
        } catch(IOException e){
            e.printStackTrace();
        }

        if(intent != null){
            startActivityForResult(intent, captureManager.REQUEST_TAKE_PHOTO);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_picker);

        if(getSupportActionBar() != null){
            getSupportActionBar().setTitle(null);
        }

        captureManager = new ImageCaptureManager(this);

        MAX_NUMBER_OF_SELECTED_IMAGES = getIntent().getIntExtra(MAX_NUMBER_OF_PICTURES, 30);
        languageCode = getIntent().getStringExtra(LANGUAGE_CODE);
        doWatermark = getIntent().getBooleanExtra(DO_WATERMARK, false);

        recyclerView = findViewById(R.id.recyler_view);
        progressBar = findViewById(R.id.progress_bar);
        imageManager = ImageManager.getInstance(this);
        mChoseAlbumsButton = findViewById(R.id.albums_button);
        mChoseAlbumsButton.setText(languageCode.equals("ar") ? "كل الصور" : "All Pictures");
        mChoseAlbumsButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                androidx.fragment.app.FragmentManager fm = getSupportFragmentManager();

                albumsDialogFragment = AlbumsDialogFragment.getInstance((ArrayList<Bucket>)buckets, allImagePaths.get(0), languageCode);
                albumsDialogFragment.show(fm, "ALBUM_DIALOG");
            }
        });

        checkWriteReadPermission();

        if(hasPermissions()){
            setupVariables();
        }

        width = getIntent().getIntExtra(WIDTH, 0);
        height = getIntent().getIntExtra(HEIGHT, 0);
        compressionRatio = getIntent().getIntExtra(RATIO, 0);
    }

    private void getProccessedImagesPaths(){
        progressBar.setVisibility(View.VISIBLE);
        ImageProcessingAsyncTask task = new ImageProcessingAsyncTask(this);
        task.execute(new ImageProcessingParams(this.width, this.height, this.compressionRatio, getSelectedImages(), this.doWatermark));
    }

    private List<String> getSelectedImages(){
        List<String> selectedImagesPaths = new ArrayList<>();
        for(Image image: selectedImages){
            selectedImagesPaths.add(image.getPath());
        }

        return selectedImagesPaths;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_image_picker, menu);

        imageSelectionMenu = menu;
        updateMenu();

        return true;
    }

    private void sendResultsBack(List<String> paths){
        Intent data = new Intent();
        ArrayList<String> selectedProcessedImagePaths = (ArrayList<String>) paths;

        if(paths.size() == 0){
            setResult(RESULT_CANCELED);
        }
        else {
            double[] imageGeoloaction = null;
            for(String path: selectedProcessedImagePaths){
                imageGeoloaction = PictureUtils.getImageLatLon(this, path);
                if(imageGeoloaction != null){
                    if(imageGeoloaction[0] != 0.0 && imageGeoloaction[1] != 0.0){
                        break;
                    }
                }
            }
            data.putExtra(RESULT_GEOLOCATION, imageGeoloaction);
        }
        data.putExtra(RESULT, selectedProcessedImagePaths);
        setResult(Activity.RESULT_OK, data);
        this.finish();
    }

    public static Intent newIntent(Context context, int width, int height, int compressionRatio, int maxNumberOfPictures, String languageCode, boolean doWatermark){
        Intent intent = new Intent(context, PickerActivity.class);
        intent.putExtra(WIDTH, width);
        intent.putExtra(HEIGHT, height);
        intent.putExtra(RATIO, compressionRatio);
        intent.putExtra(MAX_NUMBER_OF_PICTURES, maxNumberOfPictures);
        intent.putExtra(LANGUAGE_CODE, languageCode);
        intent.putExtra(DO_WATERMARK, doWatermark);
        return intent;
    }

    private void setupVariables(){
        allImagePaths = imageManager.getAllImagePaths(this);
        buckets = imageManager.getAllBuckets(this);
        for(Bucket bucket: buckets){
            bucket.setThumbnailPath(imageManager.getFirstImagePathFromBucket(getBaseContext(), bucket.getBucketName()));
        }
        imageObjectsMap = imageManager.getImageObjectsMap(this);
        updateImageObjects(null);
        selectedImages = new LinkedHashSet<>();
        mImagePickerAdapter = new ImagePickerAdapter();
        recyclerView.setAdapter(mImagePickerAdapter);
        recyclerView.setLayoutManager(new GridLayoutManager(this, 3));
    }

    private void updateUiWithBucket(String bucketName) {
        if(bucketName == null || bucketName.equals("")){
            updateImageObjects(null);
            mChoseAlbumsButton.setText(languageCode.equals("ar") ? "كل الصور" : "All Pictures");
        } else{
            updateImageObjects(bucketName);
            mChoseAlbumsButton.setText(bucketName);
        }

        mImagePickerAdapter.notifyDataSetChanged();
    }

    //Ask for permission
    private void checkWriteReadPermission(){
        boolean writePermissionGranted = ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED;
        boolean readPermissionGranted = ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED;
        if(!writePermissionGranted || !readPermissionGranted){
            String[] permissions = new String[] {Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE};
            requestStoragePermission(permissions);
        }
    }

    private boolean hasPermissions(){
        String[] permissions = new String[] {Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE};
        for(String permission: permissions){
            boolean isPermissionGranted =  ContextCompat.checkSelfPermission(this, permission) == PackageManager.PERMISSION_GRANTED;
            if(!isPermissionGranted){
                return false;
            }
        }

        return true;
    }

    private void requestStoragePermission(String[] permissions){
        ActivityCompat.requestPermissions(this, permissions, WRITE_READ_PERMISSION_REQUEST_CODE);
    }

    private void requestCameraPermission(){
        String cameraPermission = Manifest.permission.CAMERA;
        if(ContextCompat.checkSelfPermission(this, cameraPermission) == PackageManager.PERMISSION_GRANTED){
            openCamera();
            return;
        }
        ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.CAMERA}, CAMERA_PERMISSION_REQUEST_CODE);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch(requestCode){
            case WRITE_READ_PERMISSION_REQUEST_CODE:
                if(grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED){
                    setupVariables();
                    updateMenu();
                }  else if(grantResults[0] == PackageManager.PERMISSION_DENIED || grantResults[1] == PackageManager.PERMISSION_DENIED){
                    this.finish();
                }
                break;
            case CAMERA_PERMISSION_REQUEST_CODE:
                if(grantResults[0] == PackageManager.PERMISSION_GRANTED){
                    openCamera();
                } else {
                    // show message
                }
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == captureManager.REQUEST_TAKE_PHOTO && resultCode == RESULT_OK){
            if(captureManager == null){
                captureManager = new ImageCaptureManager(this);
            }
            captureManager.galleryAddPic();

            imageObjects.add(0 ,new Image(captureManager.getCurrentPhotoPath(), false));
            mImagePickerAdapter.notifyDataSetChanged();
        }
    }

    private void updateMenu(){
        MenuItem numberOfSelectedImagesItem = imageSelectionMenu.findItem(R.id.number_of_selected_images);
        MenuItem takePictureMenuItem = imageSelectionMenu.findItem(R.id.take_picture_menu_item);

        if(hasPermissions()){
            Resources resources = getResources();
            String formattedTitle = String.format(resources.getString(R.string.selected_images), languageCode.equals("ar") ? "تم" : "OK", selectedImages.size(), MAX_NUMBER_OF_SELECTED_IMAGES);

            numberOfSelectedImagesItem.setTitle(formattedTitle);
            numberOfSelectedImagesItem.setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {

                @Override
                public boolean onMenuItemClick(MenuItem menuItem) {
                    menuItem.setEnabled(false);
                    getProccessedImagesPaths();
                    return true;
                }
            });
        } else {
            numberOfSelectedImagesItem.setTitle(getResources().getString(R.string.selected_images, languageCode.equals("ar") ? "تم" : "OK", 0, MAX_NUMBER_OF_SELECTED_IMAGES));
        }

        takePictureMenuItem.setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {
            @Override
            public boolean onMenuItemClick(MenuItem menuItem) {
                requestCameraPermission();
                return true;
            }
        });
    }

    private void updateImageObjects(String bucketName){
        if(bucketName == null){
            imageObjects = imageManager.getImageObjectsFromPaths(this, imageManager.getAllImagePaths(this));
        } else {
            imageObjects = imageManager.getImageObjectsFromPaths(this, imageManager.getImagePathsFromBucket(this, bucketName));
        }
    }

    private class ImagePickerAdapter extends RecyclerView.Adapter<ImagePickerAdapter.ImagePickerViewHolder>{
        @NonNull
        @Override
        public ImagePickerViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            View imageViewHolderView = inflater.inflate(R.layout.image_picker_view_holder, parent, false);

            return new ImagePickerViewHolder(imageViewHolderView);
        }

        @Override
        public void onBindViewHolder(@NonNull ImagePickerViewHolder holder, int position) {
            Image image = imageObjects.get(position);
            holder.bind(image);
        }

        @Override
        public int getItemCount() {
            return imageObjects.size();
        }
        class ImagePickerViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener{

            Image mImage;
            ImageView imageView;
            CheckBox checkBox;

            public ImagePickerViewHolder(View itemView) {
                super(itemView);

                itemView.setOnClickListener(this);

                imageView = itemView.findViewById(R.id.image_view);
                checkBox = itemView.findViewById(R.id.image_checked);
            }

            public void bind(Image image){
                this.mImage = image;
                for(Image img: selectedImages){
                    if(img.getPath().equals(image.getPath())){
                        mImage.setSelected(img.isSelected());
                    }
                }
                updateItem(mImage.getPath(), mImage.isSelected());
            }

            public void updateItem(String imagePath, boolean checked){
                Picasso.get().load("file://"+imagePath).centerCrop().fit().into(this.imageView);
                this.checkBox.setChecked(checked);
            }

            private void updateItemUI(){
                checkBox.setChecked(mImage.isSelected());
            }

            @Override
            public void onClick(View view) {
                if(selectedImages.size() < MAX_NUMBER_OF_SELECTED_IMAGES){
                    mImage.setSelected(!mImage.isSelected());
                    if(mImage.isSelected()) {
                        selectedImages.add(mImage);
                    }
                    else{
                        selectedImages.remove(mImage);
                    }
                } else {
                    mImage.setSelected(false);
                    for(Iterator<Image> iterator = selectedImages.iterator();iterator.hasNext();){
                        Image img = iterator.next();
                        if(img.getPath().equals(mImage.getPath())){
                            iterator.remove();
                        }
                    }
                }
                updateItemUI();
                invalidateOptionsMenu();
            }
        }
    }
}
