
package com.reactlibrary;

import android.app.Activity;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.view.View;
import android.widget.Toast;

import com.facebook.common.activitylistener.BaseActivityListener;
import com.facebook.react.ReactActivity;
import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.BaseActivityEventListener;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableNativeArray;

import java.util.ArrayList;
import java.util.List;

public class RNImagePickerModule extends ReactContextBaseJavaModule {

  private final ReactApplicationContext reactContext;

  private final int PICKER_REQUEST_CODE = 3014;
  private final String RESULT = "RESULT";
    private static final String RESULT_GEOLOCATION = "RESULT_GEOLOCATION";
    private final String WIDTH = "WIDTH";
    private final String HEIGHT = "HEIGHT";
    private final String RATIO = "RATIO";
    private final String MAX_NUMBER_OF_PICTURES = "MAX";

  private Callback successCallback;

  private int width;
  private int height;
  private int maxNumberOfPictures;
  private int compressionRatio;

  private final ActivityEventListener pickerActivityListener = new BaseActivityEventListener() {
    @Override
    public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent data) {
          if(requestCode == PICKER_REQUEST_CODE && resultCode == Activity.RESULT_OK){
              ArrayList<String> paths = (ArrayList<String>) data.getSerializableExtra(RESULT);
              double[] geolocationData = data.getDoubleArrayExtra(RESULT_GEOLOCATION);
              WritableArray pathArray = getPathsWritableArray(paths);
              if(geolocationData != null){
                  successCallback.invoke(pathArray, geolocationData[0], geolocationData[1]);
              } else{
                  successCallback.invoke(pathArray, null, null);
              }
          } else if(requestCode == PICKER_REQUEST_CODE && resultCode != Activity.RESULT_OK){
              successCallback.invoke(0);
          }
      }
  };

  public RNImagePickerModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
    reactContext.addActivityEventListener(pickerActivityListener);
  }

  @ReactMethod
  public void initatePicker(int width, int height, int compressionRatio, int maxNumberOfPictures, String languageCode, boolean doWatermark, Callback sc){
      successCallback = sc;
      Intent intent = PickerActivity.newIntent(getCurrentActivity(), width, height, compressionRatio, maxNumberOfPictures, languageCode, doWatermark);
      reactContext.startActivityForResult(intent, PICKER_REQUEST_CODE, null);
  }

  private WritableArray getPathsWritableArray(List<String> paths){
      WritableArray pathArray = new WritableNativeArray();

      for(String path: paths){
          pathArray.pushString(path);
      }

      return pathArray;
  }

  @ReactMethod
  public void showT(String s){
    Toast.makeText(reactContext.getApplicationContext(), s, Toast.LENGTH_LONG).show();
  }

    @Override
  public String getName() {
    return "RNImagePicker";
  }
}